package com.xhkjedu.sapi.service.notice;

import com.xhkjedu.sapi.listener.MessageSender;
import com.xhkjedu.sapi.mapper.notice.NoticeClassMapper;
import com.xhkjedu.sapi.mapper.notice.NoticeMapper;
import com.xhkjedu.sapi.mapper.notice.NoticeUserMapper;
import com.xhkjedu.sapi.model.notice.TNoticeClass;
import com.xhkjedu.sapi.model.notice.TNoticeUser;
import com.xhkjedu.sapi.mapper.notice.*;
import com.xhkjedu.sapi.model.notice.TNoticeFile;
import com.xhkjedu.sapi.model.notice.TNotice;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.sapi.vo.notice.NoticeVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname NoticeService
 * @description 
 * @date 2020/6/12 17:55
 **/
@Service
public class NoticeService {
    @Resource
    private NoticeMapper noticeMapper;
    @Resource
    private NoticeClassMapper noticeClassMapper;
    @Resource
    private NoticeUserMapper noticeUserMapper;
    @Resource
    private NoticeFileMapper noticeFileMapper;
    @Resource
    private MessageSender messageSender;

    //添加
    @Transactional(rollbackFor = Exception.class)
    public Integer save(TNotice model) {
        if (N_Utils.isEmptyInteger(model.getCreatetime())) {
            model.setCreatetime(N_Utils.getSecondTimestamp());
        }
        noticeMapper.insertUseGeneratedKeys(model);
        model.setNoticeid(model.getId());
        //添加通知对象
        saveNoticeObj(model);

        //添加通知附件
        saveFile(model);
        return model.getId();
    }

    /**
     * @Description 添加通知附件
     * @Date 2023/12/20 9:33
     * @Author YWX
     * @Param [model]
     * @Return void
     **/
    private void saveFile(TNotice model) {
        List<TNoticeFile> files = model.getFiles();
        if (N_Utils.isListNotEmpty(files)) {
            Integer noticeid = model.getNoticeid();
            for (TNoticeFile file : files) {
                file.setNoticeid(noticeid);
            }
            noticeFileMapper.insertList(files);
        }
    }

    //删除
    public void deleteById(Integer noticeid) {
        List<TNoticeFile> files = noticeFileMapper.listByNoticeId(noticeid);
        noticeMapper.deleteByPrimaryKey(noticeid);
        resourceDel(files);
    }

    /**
     * @Description 删除服务器文件
     * @Date 2023/12/20 9:48
     * @Author YWX
     * @Param [files]
     * @Return void
     **/
    private void resourceDel(List<TNoticeFile> files) {
        if (N_Utils.isListNotEmpty(files)) {
            List<String> resourceUrls = new ArrayList<>();
            for (TNoticeFile file : files) {
                resourceUrls.add(file.getResourceurl());
                String resourcecover = file.getResourcecover();
                if (N_Utils.isNotEmpty(resourcecover)) resourceUrls.add(resourcecover);
                String pdfurl = file.getPdfurl();
                if (N_Utils.isNotEmpty(pdfurl)) resourceUrls.add(pdfurl);
            }
            messageSender.resourceDel(resourceUrls);//给文件服务器发送mq删除文件
        }
    }

    //修改
    @Transactional(rollbackFor = Exception.class)
    public void update(TNotice model) {
        noticeMapper.updateByPrimaryKeySelective(model);
        Integer noticeid = model.getNoticeid();
        //添加通知对象
        noticeUserMapper.deleteByNoticeId(noticeid);//删除通知对象
        saveNoticeObj(model);//重新添加

        //添加通知附件
        List<TNoticeFile> oldFiles = noticeFileMapper.listByNoticeId(noticeid);
        noticeFileMapper.deleteByNoticeId(noticeid);//删除通知附件
        saveFile(model);//重新添加
        //删除的文件从服务器删除
        List<TNoticeFile> delFiles = new ArrayList<>();
        List<String> resourceurls = model.getFiles().stream().map(TNoticeFile::getResourceurl).collect(Collectors.toList());
        for (TNoticeFile oldFile : oldFiles) {
            String resourceurl = oldFile.getResourceurl();
            if(!resourceurls.contains(resourceurl)){
                delFiles.add(oldFile);
            }
        }
        resourceDel(delFiles);
    }

    //添加通知对象
    private void saveNoticeObj(TNotice model) {
        model.setUsers(N_Utils.listDistinct(model.getUsers()));//通知对象去重
        noticeUserMapper.batchSave(model);
    }

    //获取详情
    public TNotice findById(Integer noticeid) {
        TNotice notice = noticeMapper.findById(noticeid);
        notice.setFiles(noticeFileMapper.listByNoticeId(noticeid));
        return notice;
    }

    //获取列表
    public List <NoticeVo> listAll(TNotice notice) {
        return noticeMapper.listAll(notice);
    }

    /**
     * 用户通知列表
     * @Param [notice]
     * @Author ywx
     * @Date 2020/6/13 9:55
     * @return java.util.List<com.xhkjedu.vo.notice.NoticeVo>
     **/
    public List<NoticeVo> listMy(TNotice notice) {
        return noticeMapper.listMy(notice);
    }

    /**
     * 学生通知列表
     * @Param [notice]
     * @Author ywx
     * @Date 2020/6/22 10:02
     * @return java.util.List<com.xhkjedu.vo.notice.NoticeVo>
     **/
    public List<NoticeVo> listStudent(TNotice notice) {
        return noticeMapper.listStudent(notice);
    }

    /**
     * 用户查看
     * @Param [noticeid, createid]
     * @Author ywx
     * @Date 2020/6/13 11:21
     * @return com.xhkjedu.model.notice.TNotice
     **/
    public TNotice reade(Integer noticeid, Integer createid) {
        TNotice notice = noticeMapper.findById2(noticeid);
        notice.setFiles(noticeFileMapper.listByNoticeId(noticeid));
        noticeUserMapper.reade(noticeid, createid, N_Utils.getSecondTimestamp());
        return notice;
    }

    /**
     * 学生查看
     * @Param [noticeid]
     * @Author ywx
     * @Date 2020/6/22 11:00
     * @return com.xhkjedu.model.notice.TNotice
     **/
    public TNotice findById2(Integer noticeid) {
        return noticeMapper.findById2(noticeid);
    }

    /**
     * 用户未看通知数量
     * @Param [teacherid]
     * @Author ywx
     * @Date 2020/7/24 10:37
     * @return java.lang.Integer
     **/
    public Integer noReadNum(Integer teacherid) {
        return noticeMapper.noReadNum(teacherid);
    }

    //通知学生--网页--列表
    public List<NoticeVo> listNoticeStuWeb(TNotice notice) {
        return noticeMapper.listNoticeForStuWeb(notice);
    }

    /**
     * @Description 通知对象
     * @Date 2023/12/18 16:26
     * @Author YWX
     * @Param [noticeid]
     * @Return java.util.List<java.util.Map>
     **/
    public List<Map> listUser(Integer noticeid) {
        return noticeUserMapper.listByNoticeId(noticeid);
    }

    /*
     * @Description 处理班级学生通知
     * @Date 2023/12/18 16:10:36
     * @Author WN
     * @Param []
     * @Return void
     **/
    public void chandleNoticeUser() {
        //获取需要处理的通知
        List<TNoticeClass> noticeClasses = noticeClassMapper.listAll();
        for (TNoticeClass noticeClass : noticeClasses) {
            List<TNoticeUser> noticeUsers = noticeClassMapper.listClassStudent(noticeClass.getClassid());
            if (N_Utils.isListNotEmpty(noticeUsers)) {
                for (TNoticeUser user : noticeUsers) {
                    user.setNoticeid(noticeClass.getNoticeid());
                    user.setReaded(1);
                    user.setCreatetime(noticeClass.getCreatetime());
                }
                noticeUserMapper.insertList(noticeUsers);
            }
        }
    }
}
